#!/usr/bin/python
# -*- coding: utf-8 -*-
##      collect_funcs.py
##
##
## Copyright (C) 2008 Max V. Stotsky <ms@pereslavl.ru>
##
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published
## by the Free Software Foundation; version 3 or later.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##

import gettext
import os
import re
import socket
import string
import subprocess
import time

import config
from pref import preferences

_ = gettext.gettext

class LowSpeedExeption(Exception):
	pass

def binary_fmt(value, labels, precision) :
	fmt = "%%5.%df %%s" % precision
	for label in labels[:-1] :
		if value < 1024.0:
			return fmt % (value, label)
		value /= 1024.0

	return fmt % (value, labels[-1])

def bytes_fmt(bytes_, precision=3) :
	labels = [_("B"),   _("KiB"), _("MiB"), _("GiB"), _("TiB"),
		_("PiB"), _("EiB"), _("ZiB"), _("YiB")]
	return binary_fmt(bytes_, labels, precision)

def bits_fmt(bits, precision=3) :
	labels = [_("bit"),   _("Kibit"), _("Mibit"), _("Gibit"), _("Tibit"),
		_("Pibit"), _("Eibit"), _("Zibit"), _("Yibit")]
	return binary_fmt(bits, labels, precision)

def estimate (x, excel, good, not_so_bad) :
	if x < excel :
		return 3
	elif x < good :
		return 2
	elif x < not_so_bad :
		return 1
	else :
		return 0

def get_user_data() :
	"""
	This function gets the data from the local file or from '/etc/passwd'.
	Function may get the person's name and phone number from '/etc/passwd'.
	"""
	user_data = {
			"yes"		  : "0",
			"name"		 : "",
			"phone_number" : "",
			"address"	  : "",
			"date"		 : "",
			"time_type"	: "0",
			"time_h1"	  : "0",
			"time_m1"	  : "0",
			"time_h2"	  : "0",
			"time_m2"	  : "0",
	}

	if not config.readconfig(preferences.opt["config_user_data"], user_data) :
		# trying to open passwd file
		fpass = None
		try :
			fpass = open("/etc/passwd")
			[name, room, homePhone, workPhone] = \
			filter(lambda s : s[0]==os.environ["USER"],
				map(lambda s : s.split(":"), fpass.readlines())
			)[0][4].split(",")

			user_data["name"] = name
			if homePhone == "" :
				user_data["phone_number"] = workPhone
			else :
				user_data["phone_number"] = workPhone
		except :
			pass
		finally :
			if fpass : fpass.close()
	return user_data

def save_user_data(user_data) :
	"""
	This function saves the data about user to the local file.
	"""
	if not os.path.exists(preferences.opt["config_user_dir"]) :
		os.makedirs(preferences.opt["config_user_dir"])
	config.saveconfig(preferences.opt["config_user_data"], user_data)

def get_speed_discard(host) :
	"""
	This function gets data rate using DISCARD service.
	"""
	addr = (host,
			preferences.opt["speed_discard_port"])
	try :
		size1 = preferences.opt["speed_size1"]
		size2 = preferences.opt["speed_size2"]

		s = socket.socket(socket.AF_INET)
		s.settimeout(3)
		s.connect(addr)
		s.settimeout(None)

		data = "a" * size1
		b = time.time()
		bytes = s.send(data)
		total_time = time.time() - b
		if total_time > 1.0 :
			raise LowSpeedExeption, "< "+_("%s/s") % bytes_fmt(bytes)

		data = "a" * size2
		b = time.time()
		bytes = s.send(data)
		total_time = time.time() - b
	except LowSpeedExeption, message :
		res = message
	except :
		res = _("n/a")
	else :
		if total_time != 0:
			res = _("%s/s") % bytes_fmt(bytes / total_time)
		else :
			res = _("n/a")
	finally :
		s.close()

	return res

def get_speed_iperf(host) :
	"""
	This function gets data rate using program iperf.
	"""
	if host == "" :
		return _("n/a")

	cmd = "iperf -t 3 -f K -c %s" % host
	regexp = ".* (\S+) KBytes/sec"

	p = subprocess.Popen(cmd, shell=True, stdout=subprocess.PIPE)
	sts = os.waitpid(p.pid, 0)

	if sts[1] == 32512 : # command not found
		p.stdout.close()
		return "iperf not found"
	elif sts[1] != 0 :
		return _("n/a")

	for strg in p.stdout :
		regres = re.search(regexp, strg)
		if regres is not None :
			return _("%s/s") % bytes_fmt(float(regres.group(1)) * 1000)

	return _("n/a")

def get_information_about_os() :
	try :
		### OS version
		f = open("/proc/version")
		res = ":OS:%s" % f.readline()
		f.close()
		### mem info
		f = open("/proc/meminfo")
		for strg in f :
			try :
				[name, val] = strg.split(":")
				val = int(val.replace("kB", ""))
			except :
				continue
			if name == "MemTotal" :
				mem_total = val
			elif name == "MemFree" :
				mem_free = val
			elif name == "Buffers" :
				mem_buff = val
			elif name == "Cached" :
				mem_cached = val
		f.close()
		mem_avail = mem_total - (mem_free + mem_buff + mem_cached)
		res += ":Total physical memory:%s\n" % bytes_fmt(mem_total * 1024)
		res += ":Avail physical memory:%s\n" % bytes_fmt(mem_avail * 1024)
		### cpu info
		cpus = []
		mhz = 0
		f = open("/proc/cpuinfo")
		for strg in f :
			try :
				[name, val] = strg.split(":")
				name = name.replace("\t","")
			except :
				continue
			if name == "processor" :
				cpus.append(val)
			if name == "cpu MHz" :
				mhz += float(val)

		f.close()
		nofcpus = len(cpus)
		res += ":CPU frequency:%.3f Mhz\n" % (mhz/nofcpus)
		if nofcpus > 1 :
			res += ":Number of CPUs:%d\n" %nofcpus
		return (True, res)
	except :
		return (False, _("Information about your system not available\n"))

def get_information_about_interfaces() :
	# get domainname and hostname
	res = ""
	res += \
		":Domain:%s\n" % preferences.opt["domain"] +\
		":Hostname:%s\n\n" % preferences.opt["host"]

	# get dns servers
	f = open("/etc/resolv.conf")
	dnss = ""
	for strg in f :
		a = filter(lambda s : s!= "", strg.split(" "))
		if a[0] == "nameserver" :
			dnss += string.join(a[1:], ",").replace("\n", "")
			dnss += ","
	f.close()
	dnss = dnss[:-1] # remove last ','

	try :
		# get list of network Interfaces
		f = open("/proc/net/dev")
		ifaces = f.readlines()[2:]
		f.close()
	except :
		return (False, _("Information about network interfaces not available\n"))

	ifconfig_reg = "^.*Link encap:(.*?)(HWaddr.*)?\s+inet addr:(\S+).*Mask:(\S+).*$"
	for strg in ifaces :
		(dev, ipmask, linktype, dnsout) = ("", "", "", "")
		iface_name = strg.split(":")[0].replace(" ", "")
		prefix = "/sys/class/net/" + iface_name

		try :
			f = open(prefix + "/carrier")
			s = f.read(1)
			f.close()
		except : continue

		status = "Disconnected"
		if s == "1" :
			status = "Connected"

		try :
			f = open(prefix + "/mtu")
			mtu = int(f.readline())
			f.close()
		except : continue

		if iface_name != "lo" :
			try :
				f = open(prefix + "/uevent")
				deva = filter(lambda s : s.split("=")[0]=="PHYSDEVPATH", f.readlines())
				f.close()
			except : continue

			if deva != [] :
				ids = deva[0].split("/")
				n = 3
				if len(ids) > 4 :
					n = 4
				id_ = ids[n][-8:-1]
				try :
					f = os.popen("lspci -s \"%s\"" % id_)
					dev = (f.readline()[:-1]).replace(id_, "")
					f.close()
				except : continue
		else :
			dev = "localhost"

		try :
			f = os.popen("/sbin/ifconfig \"%s\"" % iface_name)
			output = string.join(f.readlines(), "").replace("\n", " ")
			f.close()
		except : continue

		resexpr = re.search(ifconfig_reg, output)
		if resexpr is not None :
			linktype = resexpr.group(1)
			ipmask = "%s mask %s" % (resexpr.group(3), resexpr.group(4))

		if iface_name != "lo" :
			dnsout = dnss

		# TODO: detect speed
		# TODO: detect DHCP
		res += \
			":Adapter:%s\n" % dev +\
			":Status:%s\n" % status +\
			":MTU size:%s\n" % mtu +\
			":Interface name:%s\n" % iface_name +\
			":Interface type:%s\n" % linktype +\
			":Speed:\n" +\
			":IP address:%s\n" % ipmask +\
			":Default gateway:%s\n" % preferences.opt["speed_gateway"] +\
			":DHCP server:\n" +\
			":DNS servers:%s\n\n" % dnsout
	return (True, res)

