#!/usr/bin/python
# -*- coding: utf-8 -*-
##      speedtestnet.py
##
##
## Copyright (C) 2009 Max V. Stotsky <ms@pereslavl.ru>
##
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published
## by the Free Software Foundation; version 3 or later.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##

import math
import os
import random
import socket
import time
import urllib
import xml.parsers.expat

import debug
from pref import preferences

class SpeedTestNet(debug.Debug) :
	def __init__(self) :
		debug.Debug.__init__(self)

		self.config_url = preferences.opt["speedtest_config_url"]
		self.config_xml = ""
		self.client = {}
		self.servers = []

	def get_config(self) :
		"""
		This function gets the speedtest.net configuration file with list of
		servers.
		"""
		socket.setdefaulttimeout(10)
		config_xml_fh = urllib.urlopen(self.config_url)
		socket.setdefaulttimeout(None)

		self.config_xml = config_xml_fh.read()
		config_xml_fh.close()

	def parse_config(self) :
		"""
		This function parses the speedtest.net configuration file.
		"""
		p = xml.parsers.expat.ParserCreate()
		p.StartElementHandler = self.start_element

		p.Parse(self.config_xml)

	def start_element(self, name, attrs):
		"""
		This function is handler for xml.parsers.expat. It initializes
		self.servers and self.client.
		self.client --- parameters (ip, lat, lon, isp and other) of the user
						computer or gateway (in case NAT).
		self.servers --- list of parameters (url (for upload), lat, lon, name,
						 country, countrycode, sponsor, sponsorurl and other)
						 of remote servers.
		"""
		if name == "client":
			self.client = attrs
		elif name == "server" :
			self.servers.append(attrs)

	def servers_dist(self, s1, s2) :
		"""
		Function calculates distance from the server s1 to the server s2.
		Returns approximate distance in kilometers.
		"""
		c = preferences.const["earth_radius"]

		deg2rad = lambda alpha : alpha * math.pi / 180

		phi1 = deg2rad(float(s1["lon"]))
		psi1 = deg2rad(float(s1["lat"]))
		phi2 = deg2rad(float(s2["lon"]))
		psi2 = deg2rad(float(s2["lat"]))

		cos_psi1 = math.cos(psi1)
		x1 = c * math.cos(phi1) * cos_psi1
		y1 = c * math.sin(phi1) * cos_psi1
		z1 = c * math.sin(psi1)

		cos_psi2 = math.cos(psi2)
		x2 = c * math.cos(phi2) * cos_psi2
		y2 = c * math.sin(phi2) * cos_psi2
		z2 = c * math.sin(psi2)

		hypot3 = lambda dx, dy, dz : math.sqrt(dx*dx + dy*dy + dz*dz)

		d = hypot3(x2-x1, y2-y1, z2-z1)
		dist = c * math.acos(1 - (d*d) / (2*c*c))

		s2["dist"] = dist
		return dist

	def servers_sort_by_dist(self) :
		"""
		This function sorts self.servers by the distance from self.client.
		"""
		self.servers.sort(
			cmp=lambda s1, s2: cmp(
					self.servers_dist(self.client, s1),
					self.servers_dist(self.client, s2)
			)
		)

	def latency_test(self, server) :
		"""
		This function does the latency test.
		Returns the time in seconds (float).
		"""
		lat_url = os.path.dirname(self.servers[server]["url"]) +\
				preferences.opt["speedtest_latency_path"]
		get_data = urllib.urlencode({"x" : random.getrandbits(30)})

		b = time.time()
		latency_data_fh = urllib.urlopen("%s?%s" % (lat_url, get_data))

		data = latency_data_fh.read()
		e = time.time()

		return e - b

	def download_test(self, server) :
		"""
		This function does the download test.
		Returns the pair (bytes, seconds).
		"""
		down_url = os.path.dirname(self.servers[server]["url"]) +\
				preferences.opt["speedtest_down_data_path"]
		get_data = urllib.urlencode({"x" : random.getrandbits(30)})

		b = time.time()
		download_data_fh = urllib.urlopen("%s?%s" % (down_url, get_data))
		data = download_data_fh.read()
		e = time.time()

		return (len(data), e - b)

	def upload_test(self, server) :
		"""
		This function does the upload test.
		Returns the pair (bytes, seconds).
		"""
		up_url = self.servers[server]["url"]

		data = "a" * preferences.opt["speedtest_up_data_size"]
		post_data = urllib.urlencode({"content0" : data})

		get_data = urllib.urlencode({"x" : random.getrandbits(30) })

		b = time.time()
		fh = urllib.urlopen(up_url+"?%s" % get_data, post_data)
		data = fh.read()
		e = time.time()

		return (len(post_data), e - b)


if __name__ == "__main__":
	stn = SpeedTestNet()
	stn.get_config()
	stn.parse_config()
	stn.servers_sort_by_dist()
	print "---"
	print stn.client["isp"]

	lat = stn.latency_test(0)
	print lat

	(down_bytes, down_time) = stn.download_test(0)
	print (down_bytes / (down_time - lat)) * 8 / 1024

	(up_bytes, up_time) = stn.upload_test(0)
	print (up_bytes / (up_time - lat)) * 8 / 1024

